@@ -4,16 +4,16 @@
  * Copyright (C) 2008, Nokia
  *
  * This library is free software; you can redistribute it and/or
- * modify it under the terms of the GNU General Public
+ * modify it under the terms of the GNU Lesser General Public
  * License as published by the Free Software Foundation; either
- * version 2 of the License, or (at your option) any later version.
+ * version 2.1 of the License, or (at your option) any later version.
  *
  * This library is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * General Public License for more details.
+ * Lesser General Public License for more details.
  *
- * You should have received a copy of the GNU General Public
+ * You should have received a copy of the GNU Lesser General Public
  * License along with this library; if not, write to the
  * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
  * Boston, MA  02110-1301, USA.
@@ -28,19 +28,16 @@
 
 #include <stdio.h>
 #include <glib.h>
+#include <gio/gio.h>
 
 FILE*    tracker_file_open                         (const gchar  *uri,
 						    const gchar  *how,
 						    gboolean      sequential);
 void     tracker_file_close                        (FILE         *file,
 						    gboolean      need_again_soon);
-gboolean tracker_file_unlink                       (const gchar  *uri);
 goffset  tracker_file_get_size                     (const gchar  *uri);
 guint64  tracker_file_get_mtime                    (const gchar  *uri);
-gchar *  tracker_file_get_mime_type                (const gchar  *uri);
-void     tracker_file_get_path_and_name            (const gchar  *uri,
-						    gchar       **path,
-						    gchar       **name);
+gchar *  tracker_file_get_mime_type                (GFile        *file);
 gboolean tracker_file_system_has_enough_space      (const gchar  *path,
 						    gulong        required_bytes);
 void     tracker_path_remove                       (const gchar  *uri);
