@@ -4,16 +4,16 @@
  * Copyright (C) 2008, Nokia
  *
  * This library is free software; you can redistribute it and/or
- * modify it under the terms of the GNU General Public
+ * modify it under the terms of the GNU Lesser General Public
  * License as published by the Free Software Foundation; either
- * version 2 of the License, or (at your option) any later version.
+ * version 2.1 of the License, or (at your option) any later version.
  *
  * This library is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * General Public License for more details.
+ * Lesser General Public License for more details.
  *
- * You should have received a copy of the GNU General Public
+ * You should have received a copy of the GNU Lesser General Public
  * License along with this library; if not, write to the
  * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
  * Boston, MA  02110-1301, USA.
@@ -41,10 +41,6 @@ gchar *  tracker_gint_to_string		       (gint	      i);
 gchar *  tracker_guint_to_string	       (guint	      i);
 gchar *  tracker_gint32_to_string	       (gint32	      i);
 gchar *  tracker_guint32_to_string	       (guint32       i);
-gchar *  tracker_gint64_to_string              (gint64        i);
-gchar *  tracker_guint64_to_string             (guint64       i);
-gchar *  tracker_goffset_to_string             (goffset       i);
-
 gboolean tracker_string_to_uint		       (const gchar  *s,
 						guint	     *ret);
 gint	 tracker_string_in_string_list	       (const gchar  *str,
