@@ -4,16 +4,16 @@
  * Copyright (C) 2008, Nokia
  *
  * This library is free software; you can redistribute it and/or
- * modify it under the terms of the GNU General Public
+ * modify it under the terms of the GNU Lesser General Public
  * License as published by the Free Software Foundation; either
- * version 2 of the License, or (at your option) any later version.
+ * version 2.1 of the License, or (at your option) any later version.
  *
  * This library is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * General Public License for more details.
+ * Lesser General Public License for more details.
  *
- * You should have received a copy of the GNU General Public
+ * You should have received a copy of the GNU Lesser General Public
  * License along with this library; if not, write to the
  * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
  * Boston, MA  02110-1301, USA.
@@ -33,6 +33,26 @@ tracker_is_empty_string (const char *str)
 	return str == NULL || str[0] == '\0';
 }
 
+inline gboolean
+tracker_is_blank_string (const char *str)
+{
+	register gint len, i;
+
+	if (str == NULL || str[0] == '\0') {
+		return TRUE;
+	}
+
+	len = strlen (str);
+
+	for (i = 0; i < len; i++) {
+		if (str[i] != ' ') {
+			return FALSE;
+		}
+	}
+
+	return TRUE;
+}
+
 /* Removes a substring modifing haystack in place */
 gchar *
 tracker_string_remove (gchar	   *haystack,
@@ -102,27 +122,6 @@ tracker_string_replace (const gchar *haystack,
 	return g_string_free (str, FALSE);
 }
 
-gchar *
-tracker_escape_string (const gchar *in)
-{
-	gchar **array, *out;
-
-	if (!in) {
-		return NULL;
-	}
-
-	if (!strchr (in, '\'')) {
-		return g_strdup (in);
-	}
-
-	/* double single quotes */
-	array = g_strsplit (in, "'", -1);
-	out = g_strjoinv ("''", array);
-	g_strfreev (array);
-
-	return out;
-}
-
 gchar *
 tracker_seconds_estimate_to_string (gdouble  seconds_elapsed,
 				    gboolean short_string,
@@ -171,44 +170,36 @@ tracker_seconds_to_string (gdouble  seconds_elapsed,
 	s = g_string_new ("");
 
 	if (short_string) {
-		if (days) {
-			g_string_append_printf (s, " %dd", days);
+		if (days) { /* Translators: this is %d days */
+			g_string_append_printf (s, _(" %dd"), days);
 		}
 
-		if (hours) {
-			g_string_append_printf (s, " %2.2dh", hours);
+		if (hours) { /* Translators: this is %2.2d hours */
+			g_string_append_printf (s, _(" %2.2dh"), hours);
 		}
 
-		if (minutes) {
-			g_string_append_printf (s, " %2.2dm", minutes);
+		if (minutes) { /* Translators: this is %2.2d minutes */
+			g_string_append_printf (s, _(" %2.2dm"), minutes);
 		}
 
-		if (seconds) {
-			g_string_append_printf (s, " %2.2ds", seconds);
+		if (seconds) { /* Translators: this is %2.2d seconds */
+			g_string_append_printf (s, _(" %2.2ds"), seconds);
 		}
 	} else {
 		if (days) {
-			g_string_append_printf (s, " %d day%s",
-						days,
-						days == 1 ? "" : "s");
+			g_string_append_printf (s, ngettext (" %d day", " %d days", days), days);
 		}
 
 		if (hours) {
-			g_string_append_printf (s, " %2.2d hour%s",
-						hours,
-						hours == 1 ? "" : "s");
+			g_string_append_printf (s, ngettext (" %2.2d hour", " %2.2d hours", hours), hours);
 		}
 
 		if (minutes) {
-			g_string_append_printf (s, " %2.2d minute%s",
-						minutes,
-						minutes == 1 ? "" : "s");
+			g_string_append_printf (s, ngettext (" %2.2d minute", " %2.2d minutes", minutes), minutes);
 		}
 
 		if (seconds) {
-			g_string_append_printf (s, " %2.2d second%s",
-						seconds,
-						seconds == 1 ? "" : "s");
+			g_string_append_printf (s, ngettext (" %2.2d second", " %2.2d seconds", seconds), seconds);
 		}
 	}
 
@@ -224,26 +215,6 @@ tracker_seconds_to_string (gdouble  seconds_elapsed,
 	return str;
 }
 
-void
-tracker_throttle (TrackerConfig *config,
-		  gint		 multiplier)
-{
-	gint throttle;
-
-	g_return_if_fail (TRACKER_IS_CONFIG (config));
-
-	/* Get the throttle, add 5 (minimum value) so we don't do
-	 * nothing and then multiply it by the factor given
-	 */
-	throttle  = tracker_config_get_throttle (config);
-	throttle += 5;
-	throttle *= multiplier;
-
-	if (throttle > 0) {
-		g_usleep (throttle);
-	}
-}
-
 /* Temporary: Just here until we upgrade to GLib 2.18. */
 static gboolean
 tracker_dgettext_should_translate (void)
@@ -305,3 +276,288 @@ tracker_dngettext (const gchar *domain,
 
   return dngettext (domain, msgid, msgid_plural, n);
 }
+
+
+static const char *
+find_conversion (const char  *format,
+                 const char **after)
+{
+  const char *start = format;
+  const char *cp;
+
+  while (*start != '\0' && *start != '%')
+    start++;
+
+  if (*start == '\0')
+    {
+      *after = start;
+      return NULL;
+    }
+
+  cp = start + 1;
+
+  if (*cp == '\0')
+    {
+      *after = cp;
+      return NULL;
+    }
+
+  /* Test for positional argument.  */
+  if (*cp >= '0' && *cp <= '9')
+    {
+      const char *np;
+
+      for (np = cp; *np >= '0' && *np <= '9'; np++)
+        ;
+      if (*np == '$')
+        cp = np + 1;
+    }
+
+  /* Skip the flags.  */
+  for (;;)
+    {
+      if (*cp == '\'' ||
+          *cp == '-' ||
+          *cp == '+' ||
+          *cp == ' ' ||
+          *cp == '#' ||
+          *cp == '0')
+        cp++;
+      else
+        break;
+    }
+
+  /* Skip the field width.  */
+  if (*cp == '*')
+    {
+      cp++;
+
+      /* Test for positional argument.  */
+      if (*cp >= '0' && *cp <= '9')
+        {
+          const char *np;
+
+          for (np = cp; *np >= '0' && *np <= '9'; np++)
+            ;
+          if (*np == '$')
+            cp = np + 1;
+        }
+    }
+  else
+    {
+      for (; *cp >= '0' && *cp <= '9'; cp++)
+        ;
+    }
+
+  /* Skip the precision.  */
+  if (*cp == '.')
+    {
+      cp++;
+      if (*cp == '*')
+        {
+          /* Test for positional argument.  */
+          if (*cp >= '0' && *cp <= '9')
+            {
+              const char *np;
+
+              for (np = cp; *np >= '0' && *np <= '9'; np++)
+                ;
+              if (*np == '$')
+                cp = np + 1;
+            }
+        }
+      else
+        {
+          for (; *cp >= '0' && *cp <= '9'; cp++)
+            ;
+        }
+    }
+
+  /* Skip argument type/size specifiers.  */
+  while (*cp == 'h' ||
+         *cp == 'L' ||
+         *cp == 'l' ||
+         *cp == 'j' ||
+         *cp == 'z' ||
+         *cp == 'Z' ||
+         *cp == 't')
+    cp++;
+
+  /* Skip the conversion character.  */
+  cp++;
+
+  *after = cp;
+  return start;
+}
+
+gchar *
+tracker_uri_vprintf_escaped (const gchar *format, 
+                             va_list      args)
+{
+  GString *format1;
+  GString *format2;
+  GString *result = NULL;
+  gchar *output1 = NULL;
+  gchar *output2 = NULL;
+  const char *p;
+  char       *op1, *op2;
+  va_list args2;
+
+  format1 = g_string_new (NULL);
+  format2 = g_string_new (NULL);
+  p = format;
+  while (TRUE)
+    {
+      const char *after;
+      const char *conv = find_conversion (p, &after);
+      if (!conv)
+        break;
+
+      g_string_append_len (format1, conv, after - conv);
+      g_string_append_c (format1, 'X');
+      g_string_append_len (format2, conv, after - conv);
+      g_string_append_c (format2, 'Y');
+
+      p = after;
+    }
+
+  /* Use them to format the arguments
+   */
+  G_VA_COPY (args2, args);
+
+  output1 = g_strdup_vprintf (format1->str, args);
+  va_end (args);
+  if (!output1)
+    goto cleanup;
+
+  output2 = g_strdup_vprintf (format2->str, args2);
+  va_end (args2);
+  if (!output2)
+    goto cleanup;
+
+  result = g_string_new (NULL);
+
+  op1 = output1;
+  op2 = output2;
+  p = format;
+  while (TRUE)
+    {
+      const char *after;
+      const char *output_start;
+      const char *conv = find_conversion (p, &after);
+      char *escaped;
+
+      if (!conv)
+        {
+          g_string_append_len (result, p, after - p);
+          break;
+        }
+
+      g_string_append_len (result, p, conv - p);
+      output_start = op1;
+      while (*op1 == *op2)
+        {
+          op1++;
+          op2++;
+        }
+
+      *op1 = '\0';
+      escaped = g_uri_escape_string (output_start, NULL, FALSE);
+      g_string_append (result, escaped);
+      g_free (escaped);
+
+      p = after;
+      op1++;
+      op2++;
+    }
+
+ cleanup:
+  g_string_free (format1, TRUE);
+  g_string_free (format2, TRUE);
+  g_free (output1);
+  g_free (output2);
+
+  if (result)
+    return g_string_free (result, FALSE);
+  else
+    return NULL;
+}
+
+gchar *
+tracker_uri_printf_escaped (const gchar *format, ...)
+{
+  char *result;
+  va_list args;
+
+  va_start (args, format);
+  result = tracker_uri_vprintf_escaped (format, args);
+  va_end (args);
+
+  return result;
+}
+
+
+gchar *
+tracker_coalesce (gint n_values,
+                  ...)
+{
+	va_list args;
+	gint    i;
+	gchar *result = NULL;
+
+	va_start (args, n_values);
+
+	for (i = 0; i < n_values; i++) {
+		gchar *value;
+
+		value = va_arg (args, gchar *);
+		if (value) {
+			if (!result) {
+				result = value;
+			} else {
+				g_free (value);
+			}
+		}
+	}
+
+	va_end (args);
+
+	return result;
+}
+
+
+gchar *
+tracker_merge (const gchar *delim, gint n_values,
+               ...)
+{
+	va_list args;
+	gint    i;
+	GString *str = NULL;
+
+	va_start (args, n_values);
+
+	for (i = 0; i < n_values; i++) {
+		gchar *value;
+
+		value = va_arg (args, gchar *);
+		if (value) {
+			if (!str) {
+				str = g_string_new (value);
+			} else {
+				if (delim) {
+					g_string_append (str, delim);
+				}
+				g_string_append (str, value);
+			}
+			g_free (value);
+		}
+	}
+
+	va_end (args);
+
+	if (!str) {
+		return NULL;
+	}
+
+	return g_string_free (str, FALSE);
+}
