@@ -35,6 +35,8 @@
 #include "tracker-miner-files.h"
 #include "tracker-config.h"
 #include "tracker-extract-client.h"
+#include "tracker-thumbnailer.h"
+#include "tracker-marshal.h"
 
 #define DISK_SPACE_CHECK_FREQUENCY 10
 
@@ -123,8 +125,12 @@ static void     low_disk_space_limit_cb       (GObject              *gobject,
 					       GParamSpec           *arg1,
 					       gpointer              user_data);
 
-static DBusGProxy * create_extractor_proxy    (void);
-
+static DBusGProxy * extractor_create_proxy    (void);
+static void    extractor_queue_thumbnail_cb   (DBusGProxy           *proxy,
+					       const gchar          *filename, 
+					       const gchar          *mime_type,
+					       gpointer              user_data);
+	
 static gboolean miner_files_check_file        (TrackerMinerFS       *fs,
 					       GFile                *file);
 static gboolean miner_files_check_directory   (TrackerMinerFS       *fs,
@@ -173,7 +179,6 @@ tracker_miner_files_class_init (TrackerMinerFilesClass *klass)
 	miner_files_error_quark = g_quark_from_static_string ("TrackerMinerFiles");
 }
 
-
 static void
 tracker_miner_files_init (TrackerMinerFiles *mf)
 {
@@ -210,8 +215,24 @@ tracker_miner_files_init (TrackerMinerFiles *mf)
 			  G_CALLBACK (mount_pre_unmount_cb),
 			  mf);
 
-	priv->extractor_proxy = create_extractor_proxy ();
+	/* Set up extractor and signals */
+	priv->extractor_proxy = extractor_create_proxy ();
 
+	dbus_g_object_register_marshaller (tracker_marshal_VOID__STRING_STRING,
+					   G_TYPE_NONE,
+					   G_TYPE_STRING,
+					   G_TYPE_STRING,
+					   G_TYPE_INVALID);
+
+	dbus_g_proxy_add_signal (priv->extractor_proxy, "QueueThumbnail",
+				 G_TYPE_STRING,
+				 G_TYPE_STRING, 
+				 G_TYPE_INVALID);
+
+	dbus_g_proxy_connect_signal (priv->extractor_proxy, "QueueThumbnail",
+				     G_CALLBACK (extractor_queue_thumbnail_cb),
+				     NULL, NULL);
+	
 	init_mount_points (mf);
 }
 
@@ -264,6 +285,12 @@ miner_files_finalize (GObject *object)
 	mf = TRACKER_MINER_FILES (object);
 	priv = mf->private;
 
+	dbus_g_proxy_disconnect_signal (priv->extractor_proxy, "QueueThumbnail",
+					G_CALLBACK (extractor_queue_thumbnail_cb),
+					NULL);
+	
+	g_object_unref (priv->extractor_proxy);
+
 	g_signal_handlers_disconnect_by_func (priv->config,
 					      low_disk_space_limit_cb,
 					      NULL);
@@ -584,7 +611,7 @@ init_mount_points (TrackerMinerFiles *miner)
 	}
 
 	g_string_free (accumulator, TRUE);
-	g_hash_table_destroy (volumes);
+	g_hash_table_unref (volumes);
 }
 
 #ifdef HAVE_HAL
@@ -778,35 +805,6 @@ mount_pre_unmount_cb (GVolumeMonitor    *volume_monitor,
 	g_object_unref (mount_root);
 }
 
-static DBusGProxy *
-create_extractor_proxy (void)
-{
-	DBusGProxy *proxy;
-	DBusGConnection *connection;
-	GError *error = NULL;
-
-	connection = dbus_g_bus_get (DBUS_BUS_SESSION, &error);
-
-	if (!connection) {
-		g_critical ("Could not connect to the D-Bus session bus, %s",
-			    error ? error->message : "no error given.");
-		g_clear_error (&error);
-		return FALSE;
-	}
-
-	/* Get proxy for the extractor */
-	proxy = dbus_g_proxy_new_for_name (connection,
-					   "org.freedesktop.Tracker1.Extract",
-					   "/org/freedesktop/Tracker1/Extract",
-					   "org.freedesktop.Tracker1.Extract");
-
-	if (!proxy) {
-		g_critical ("Could not create a DBusGProxy to the extract service");
-	}
-
-	return proxy;
-}
-
 static gboolean
 disk_space_check (TrackerMinerFiles *mf)
 {
@@ -923,11 +921,13 @@ miner_files_check_file (TrackerMinerFS *fs,
 	GFileInfo *file_info;
 	GSList *l;
 	gchar *basename;
+	gchar *path;
 	gboolean should_process;
 
 	file_info = NULL;
 	should_process = FALSE;
 	basename = NULL;
+	path = NULL;
 
 	file_info = g_file_query_info (file,
 				       G_FILE_ATTRIBUTE_STANDARD_IS_HIDDEN,
@@ -942,6 +942,14 @@ miner_files_check_file (TrackerMinerFS *fs,
 	/* Check module file ignore patterns */
 	mf = TRACKER_MINER_FILES (fs);
 
+	path = g_file_get_path (file);
+
+	for (l = tracker_config_get_ignored_file_paths (mf->private->config); l; l = l->next) {
+		if (strcmp (l->data, path) == 0) {
+			goto done;
+		}
+	}
+
 	basename = g_file_get_basename (file);
 	
 	for (l = tracker_config_get_ignored_file_patterns (mf->private->config); l; l = l->next) {
@@ -953,12 +961,13 @@ miner_files_check_file (TrackerMinerFS *fs,
 	should_process = TRUE;
 
 done:
+	g_free (basename);
+	g_free (path);
+
 	if (file_info) {
 		g_object_unref (file_info);
 	}
 
-	g_free (basename);
-
 	return should_process;
 }
 
@@ -970,6 +979,7 @@ miner_files_check_directory (TrackerMinerFS *fs,
 	GFileInfo *file_info;
 	GSList *l;
 	gchar *basename;
+	gchar *path;
 	gboolean should_process;
 
 	should_process = FALSE;
@@ -981,13 +991,13 @@ miner_files_check_directory (TrackerMinerFS *fs,
                                        G_FILE_QUERY_INFO_NOFOLLOW_SYMLINKS,
                                        NULL, NULL);
 
+	path = g_file_get_path (file);
+
 	if (file_info && g_file_info_get_is_hidden (file_info)) {
 		TrackerMinerFiles *mf;
 		GSList *allowed_directories;
-		gchar *path;
 
 		mf = TRACKER_MINER_FILES (fs);
-		path = g_file_get_path (file);
 
 		/* FIXME: We need to check if the file is actually a
 		 * config specified location before blanket ignoring
@@ -1007,14 +1017,18 @@ miner_files_check_directory (TrackerMinerFS *fs,
 			should_process = TRUE;
 		}
 
-		g_free (path);
-
 		/* Ignore hidden dirs */
 		goto done;
 	}
 
 	mf = TRACKER_MINER_FILES (fs);
 
+	for (l = tracker_config_get_ignored_directory_paths (mf->private->config); l; l = l->next) {
+		if (strcmp (l->data, path) == 0) {
+			goto done;
+		}
+	}
+
 	basename = g_file_get_basename (file);
 
 	for (l = tracker_config_get_ignored_file_patterns (mf->private->config); l; l = l->next) {
@@ -1027,12 +1041,13 @@ miner_files_check_directory (TrackerMinerFS *fs,
 	should_process = TRUE;
 
 done:
+	g_free (basename);
+	g_free (path);
+
 	if (file_info) {
 		g_object_unref (file_info);
 	}
 
-	g_free (basename);
-
 	return should_process;
 }
 
@@ -1145,11 +1160,49 @@ process_file_data_free (ProcessFileData *data)
 	g_slice_free (ProcessFileData, data);
 }
 
+static DBusGProxy *
+extractor_create_proxy (void)
+{
+	DBusGProxy *proxy;
+	DBusGConnection *connection;
+	GError *error = NULL;
+
+	connection = dbus_g_bus_get (DBUS_BUS_SESSION, &error);
+
+	if (!connection) {
+		g_critical ("Could not connect to the D-Bus session bus, %s",
+			    error ? error->message : "no error given.");
+		g_clear_error (&error);
+		return FALSE;
+	}
+
+	/* Get proxy for the extractor */
+	proxy = dbus_g_proxy_new_for_name (connection,
+					   "org.freedesktop.Tracker1.Extract",
+					   "/org/freedesktop/Tracker1/Extract",
+					   "org.freedesktop.Tracker1.Extract");
+
+	if (!proxy) {
+		g_critical ("Could not create a DBusGProxy to the extract service");
+	}
+
+	return proxy;
+}
+
 static void
-get_embedded_metadata_cb (DBusGProxy *proxy,
-			  gchar      *sparql,
-			  GError     *error,
-			  gpointer    user_data)
+extractor_queue_thumbnail_cb (DBusGProxy  *proxy,
+			      const gchar *filename, 
+			      const gchar *mime_type,
+			      gpointer     user_data)
+{
+	tracker_thumbnailer_queue_add (filename, mime_type);
+}
+
+static void
+extractor_get_embedded_metadata_cb (DBusGProxy *proxy,
+				    gchar      *sparql,
+				    GError     *error,
+				    gpointer    user_data)
 {
 	ProcessFileData *data = user_data;
 
@@ -1174,8 +1227,8 @@ get_embedded_metadata_cb (DBusGProxy *proxy,
 }
 
 static void
-get_embedded_metadata_cancel (GCancellable    *cancellable,
-			      ProcessFileData *data)
+extractor_get_embedded_metadata_cancel (GCancellable    *cancellable,
+					ProcessFileData *data)
 {
 	GError *error;
 
@@ -1191,17 +1244,17 @@ get_embedded_metadata_cancel (GCancellable    *cancellable,
 }
 
 static void
-get_embedded_metadata (ProcessFileData *data,
-		       const gchar     *uri,
-		       const gchar     *mime_type)
+extractor_get_embedded_metadata (ProcessFileData *data,
+				 const gchar     *uri,
+				 const gchar     *mime_type)
 {
 	data->call = org_freedesktop_Tracker1_Extract_get_metadata_async (data->miner->private->extractor_proxy,
 									  uri,
 									  mime_type,
-									  get_embedded_metadata_cb,
+									  extractor_get_embedded_metadata_cb,
 									  data);
 	g_signal_connect (data->cancellable, "cancelled",
-			  G_CALLBACK (get_embedded_metadata_cancel), data);
+			  G_CALLBACK (extractor_get_embedded_metadata_cancel), data);
 }
 
 static void
@@ -1227,7 +1280,6 @@ process_file_cb (GObject      *object,
 		/* Something bad happened, notify about the error */
 		data->callback (TRACKER_MINER_FS (data->miner), file, sparql, error, data->callback_data);
 		process_file_data_free (data);
-		g_object_unref (file_info);
 		return;
 	}
 
@@ -1279,8 +1331,13 @@ process_file_cb (GObject      *object,
 
         miner_files_add_to_datasource (data->miner, file, sparql);
 
+	/* Send file/mime data to thumbnailer (which adds it to the
+	 * queue if the thumbnailer handles those mime types).
+	 */
+	tracker_thumbnailer_queue_add (uri, mime_type);
+
 	/* Next step, getting embedded metadata */
-	get_embedded_metadata (data, uri, mime_type);
+	extractor_get_embedded_metadata (data, uri, mime_type);
 
 	g_object_unref (file_info);
 	g_free (uri);
